#!/bin/bash

# ---------------------------------- #
# SHARED OPTIONS
#
# Run 'vprocessor.py' without any arguments to see the full option list (and description)
#
# Warning: there are some naming conventions that differ from the ones used in the paper and the ones used in the code
# and in the bash scripts (sh)
#
# ************************************************************************************************************
# ************************************************************************************************************
# ************************************************************************************************************
# THIS IS JUST AN EXAMPLE FOR THE VIDEO "car.avi" IN THE CASE OF TABLE 2.
# THE PURPOSE OF THIS SCRIPT IS TO SHOW THE PARAMETER LIST USED IN TABLE 2,3.
# THE CASES OF "skater.avi" and "matrix.avi" ARE EQUIVALENT TO THIS ONE.
# THE ONLY DIFFERENCE IS THE NUMBER OF REPETITIONS OF THE VIDEO (adjusted to run approx 50k frames per layer).
# ************************************************************************************************************
# ************************************************************************************************************
# ************************************************************************************************************
# ---------------------------------- #
videopath=data/
videoname=car.avi

step=0.04

features=5
filtersize=5

eta=0.0005
rho=0
blur=1

eps1=300
eps2=300
eps3=300

# multi-layer options
layers=3  # 3 layers
rep=540  # 180 repetitions per layer
c_frames_min=57960  # this is the number of frames for 180 repetitions of the input video
c_frames=57960  # this is the number of frames for 180 repetitions of the input video

lambdaC=1000
lambdaE_0=2000
lambdaE_1=5000
lambdaE_2=5000

c_eps1=1000000
c_eps2=1000000
c_eps3=1000000

# ---------------------------------- #
# EXPERIMENTS
# ---------------------------------- #

for i in `seq 1 10`;
do

outpath=tab2_tab3/${i}/
mkdir -p ${outpath}
mkdir -p ${outpath}/test

theta=0.0001
alpha=7.8125
beta=0.00000003125
gamma=0.000375
k=0.000000000000000000625

lambdaM=100

expname=lambdaM${lambdaM}"_"${videoname}"_"rep${rep}"_"m${features}"_"f${filtersize}
python ../vprocessor.py --run ${videopath}${videoname} --out ${outpath}/${expname} --port 0 --gray 1 --save_scores_only 1 --res 240x110 --day_only 0 --check_params 0 --rep ${rep} --all_black 0 --grad 0 --gew 1.0 --m ${features} --f ${filtersize} --init_fixed 0 --init_q 1.0 --alpha ${alpha} --beta ${beta} --gamma ${gamma} --k ${k} --theta ${theta} --eta ${eta} --rho ${rho} --eps1 ${eps1} --eps2 ${eps2} --eps3 ${eps3} --step_size ${step} --step_adapt 0 --lambdaM ${lambdaM} --lambdaE_0 ${lambdaE_0} --lambdaE_1 ${lambdaE_1} --lambdaE_2 ${lambdaE_2} --lambdaC ${lambdaC} --blur ${blur} --grad_order2 0 --layers ${layers} --c_frames_min ${c_frames_min} --c_frames ${c_frames} --c_eps1 ${c_eps1} --c_eps2 ${c_eps2} --c_eps3 ${c_eps3}
cp -r ${outpath}/${expname} ${outpath}/test/${expname}
python ../vprocessor.py --run ${videopath}${videoname} --out ${outpath}/test/${expname} --resume 2 --port 0 --gray 1 --save_scores_only 0 --res 240x110 --day_only 0 --check_params 0 --rep 1 --all_black 0 --grad 0 --gew 1.0 --m ${features} --f ${filtersize} --init_fixed 0 --init_q 1.0 --alpha ${alpha} --beta ${beta} --gamma ${gamma} --k ${k} --theta ${theta} --eta ${eta} --rho 1 --eps1 ${eps1} --eps2 ${eps2} --eps3 ${eps3} --step_size 0 --step_adapt 0 --lambdaM ${lambdaM} --lambdaE_0 ${lambdaE_0} --lambdaE_1 ${lambdaE_1} --lambdaE_2 ${lambdaE_2} --lambdaC ${lambdaC} --blur ${blur} --grad_order2 0 --layers ${layers} --c_frames_min ${c_frames_min} --c_frames ${c_frames} --c_eps1 ${c_eps1} --c_eps2 ${c_eps2} --c_eps3 ${c_eps3}


# ---------------------------------- #
lambdaM=1

expname=lambdaM${lambdaM}"_"${videoname}"_"rep${rep}"_"m${features}"_"f${filtersize}
python ../vprocessor.py --run ${videopath}${videoname} --out ${outpath}/${expname} --port 0 --gray 1 --save_scores_only 1 --res 240x110 --day_only 0 --check_params 0 --rep ${rep} --all_black 0 --grad 0 --gew 1.0 --m ${features} --f ${filtersize} --init_fixed 0 --init_q 1.0 --alpha ${alpha} --beta ${beta} --gamma ${gamma} --k ${k} --theta ${theta} --eta ${eta} --rho ${rho} --eps1 ${eps1} --eps2 ${eps2} --eps3 ${eps3} --step_size ${step} --step_adapt 0 --lambdaM ${lambdaM} --lambdaE_0 ${lambdaE_0} --lambdaE_1 ${lambdaE_1} --lambdaE_2 ${lambdaE_2} --lambdaC ${lambdaC} --blur ${blur} --grad_order2 0 --layers ${layers} --c_frames_min ${c_frames_min} --c_frames ${c_frames} --c_eps1 ${c_eps1} --c_eps2 ${c_eps2} --c_eps3 ${c_eps3}
cp -r ${outpath}/${expname} ${outpath}/test/${expname}
python ../vprocessor.py --run ${videopath}${videoname} --out ${outpath}/test/${expname} --resume 2 --port 0 --gray 1 --save_scores_only 0 --res 240x110 --day_only 0 --check_params 0 --rep 1 --all_black 0 --grad 0 --gew 1.0 --m ${features} --f ${filtersize} --init_fixed 0 --init_q 1.0 --alpha ${alpha} --beta ${beta} --gamma ${gamma} --k ${k} --theta ${theta} --eta ${eta} --rho 1 --eps1 ${eps1} --eps2 ${eps2} --eps3 ${eps3} --step_size 0 --step_adapt 0 --lambdaM ${lambdaM} --lambdaE_0 ${lambdaE_0} --lambdaE_1 ${lambdaE_1} --lambdaE_2 ${lambdaE_2} --lambdaC ${lambdaC} --blur ${blur} --grad_order2 0 --layers ${layers} --c_frames_min ${c_frames_min} --c_frames ${c_frames} --c_eps1 ${c_eps1} --c_eps2 ${c_eps2} --c_eps3 ${c_eps3}


# ---------------------------------- #
lambdaM=0.01

expname=lambdaM${lambdaM}"_"${videoname}"_"rep${rep}"_"m${features}"_"f${filtersize}
python ../vprocessor.py --run ${videopath}${videoname} --out ${outpath}/${expname} --port 0 --gray 1 --save_scores_only 1 --res 240x110 --day_only 0 --check_params 0 --rep ${rep} --all_black 0 --grad 0 --gew 1.0 --m ${features} --f ${filtersize} --init_fixed 0 --init_q 1.0 --alpha ${alpha} --beta ${beta} --gamma ${gamma} --k ${k} --theta ${theta} --eta ${eta} --rho ${rho} --eps1 ${eps1} --eps2 ${eps2} --eps3 ${eps3} --step_size ${step} --step_adapt 0 --lambdaM ${lambdaM} --lambdaE_0 ${lambdaE_0} --lambdaE_1 ${lambdaE_1} --lambdaE_2 ${lambdaE_2} --lambdaC ${lambdaC} --blur ${blur} --grad_order2 0 --layers ${layers} --c_frames_min ${c_frames_min} --c_frames ${c_frames} --c_eps1 ${c_eps1} --c_eps2 ${c_eps2} --c_eps3 ${c_eps3}
cp -r ${outpath}/${expname} ${outpath}/test/${expname}
python ../vprocessor.py --run ${videopath}${videoname} --out ${outpath}/test/${expname} --resume 2 --port 0 --gray 1 --save_scores_only 0 --res 240x110 --day_only 0 --check_params 0 --rep 1 --all_black 0 --grad 0 --gew 1.0 --m ${features} --f ${filtersize} --init_fixed 0 --init_q 1.0 --alpha ${alpha} --beta ${beta} --gamma ${gamma} --k ${k} --theta ${theta} --eta ${eta} --rho 1 --eps1 ${eps1} --eps2 ${eps2} --eps3 ${eps3} --step_size 0 --step_adapt 0 --lambdaM ${lambdaM} --lambdaE_0 ${lambdaE_0} --lambdaE_1 ${lambdaE_1} --lambdaE_2 ${lambdaE_2} --lambdaC ${lambdaC} --blur ${blur} --grad_order2 0 --layers ${layers} --c_frames_min ${c_frames_min} --c_frames ${c_frames} --c_eps1 ${c_eps1} --c_eps2 ${c_eps2} --c_eps3 ${c_eps3}

# ---------------------------------- #
lambdaM=0.0001

expname=lambdaM${lambdaM}"_"${videoname}"_"rep${rep}"_"m${features}"_"f${filtersize}
python ../vprocessor.py --run ${videopath}${videoname} --out ${outpath}/${expname} --port 0 --gray 1 --save_scores_only 1 --res 240x110 --day_only 0 --check_params 0 --rep ${rep} --all_black 0 --grad 0 --gew 1.0 --m ${features} --f ${filtersize} --init_fixed 0 --init_q 1.0 --alpha ${alpha} --beta ${beta} --gamma ${gamma} --k ${k} --theta ${theta} --eta ${eta} --rho ${rho} --eps1 ${eps1} --eps2 ${eps2} --eps3 ${eps3} --step_size ${step} --step_adapt 0 --lambdaM ${lambdaM} --lambdaE_0 ${lambdaE_0} --lambdaE_1 ${lambdaE_1} --lambdaE_2 ${lambdaE_2} --lambdaC ${lambdaC} --blur ${blur} --grad_order2 0 --layers ${layers} --c_frames_min ${c_frames_min} --c_frames ${c_frames} --c_eps1 ${c_eps1} --c_eps2 ${c_eps2} --c_eps3 ${c_eps3}
cp -r ${outpath}/${expname} ${outpath}/test/${expname}
python ../vprocessor.py --run ${videopath}${videoname} --out ${outpath}/test/${expname} --resume 2 --port 0 --gray 1 --save_scores_only 0 --res 240x110 --day_only 0 --check_params 0 --rep 1 --all_black 0 --grad 0 --gew 1.0 --m ${features} --f ${filtersize} --init_fixed 0 --init_q 1.0 --alpha ${alpha} --beta ${beta} --gamma ${gamma} --k ${k} --theta ${theta} --eta ${eta} --rho 1 --eps1 ${eps1} --eps2 ${eps2} --eps3 ${eps3} --step_size 0 --step_adapt 0 --lambdaM ${lambdaM} --lambdaE_0 ${lambdaE_0} --lambdaE_1 ${lambdaE_1} --lambdaE_2 ${lambdaE_2} --lambdaC ${lambdaC} --blur ${blur} --grad_order2 0 --layers ${layers} --c_frames_min ${c_frames_min} --c_frames ${c_frames} --c_eps1 ${c_eps1} --c_eps2 ${c_eps2} --c_eps3 ${c_eps3}

# ---------------------------------- #
lambdaM=0.000001

expname=lambdaM${lambdaM}"_"${videoname}"_"rep${rep}"_"m${features}"_"f${filtersize}
python ../vprocessor.py --run ${videopath}${videoname} --out ${outpath}/${expname} --port 0 --gray 1 --save_scores_only 1 --res 240x110 --day_only 0 --check_params 0 --rep ${rep} --all_black 0 --grad 0 --gew 1.0 --m ${features} --f ${filtersize} --init_fixed 0 --init_q 1.0 --alpha ${alpha} --beta ${beta} --gamma ${gamma} --k ${k} --theta ${theta} --eta ${eta} --rho ${rho} --eps1 ${eps1} --eps2 ${eps2} --eps3 ${eps3} --step_size ${step} --step_adapt 0 --lambdaM ${lambdaM} --lambdaE_0 ${lambdaE_0} --lambdaE_1 ${lambdaE_1} --lambdaE_2 ${lambdaE_2} --lambdaC ${lambdaC} --blur ${blur} --grad_order2 0 --layers ${layers} --c_frames_min ${c_frames_min} --c_frames ${c_frames} --c_eps1 ${c_eps1} --c_eps2 ${c_eps2} --c_eps3 ${c_eps3}
cp -r ${outpath}/${expname} ${outpath}/test/${expname}
python ../vprocessor.py --run ${videopath}${videoname} --out ${outpath}/test/${expname} --resume 2 --port 0 --gray 1 --save_scores_only 0 --res 240x110 --day_only 0 --check_params 0 --rep 1 --all_black 0 --grad 0 --gew 1.0 --m ${features} --f ${filtersize} --init_fixed 0 --init_q 1.0 --alpha ${alpha} --beta ${beta} --gamma ${gamma} --k ${k} --theta ${theta} --eta ${eta} --rho 1 --eps1 ${eps1} --eps2 ${eps2} --eps3 ${eps3} --step_size 0 --step_adapt 0 --lambdaM ${lambdaM} --lambdaE_0 ${lambdaE_0} --lambdaE_1 ${lambdaE_1} --lambdaE_2 ${lambdaE_2} --lambdaC ${lambdaC} --blur ${blur} --grad_order2 0 --layers ${layers} --c_frames_min ${c_frames_min} --c_frames ${c_frames} --c_eps1 ${c_eps1} --c_eps2 ${c_eps2} --c_eps3 ${c_eps3}

# ---------------------------------- #
lambdaM=0.00000001

expname=lambdaM${lambdaM}"_"${videoname}"_"rep${rep}"_"m${features}"_"f${filtersize}
python ../vprocessor.py --run ${videopath}${videoname} --out ${outpath}/${expname} --port 0 --gray 1 --save_scores_only 1 --res 240x110 --day_only 0 --check_params 0 --rep ${rep} --all_black 0 --grad 0 --gew 1.0 --m ${features} --f ${filtersize} --init_fixed 0 --init_q 1.0 --alpha ${alpha} --beta ${beta} --gamma ${gamma} --k ${k} --theta ${theta} --eta ${eta} --rho ${rho} --eps1 ${eps1} --eps2 ${eps2} --eps3 ${eps3} --step_size ${step} --step_adapt 0 --lambdaM ${lambdaM} --lambdaE_0 ${lambdaE_0} --lambdaE_1 ${lambdaE_1} --lambdaE_2 ${lambdaE_2} --lambdaC ${lambdaC} --blur ${blur} --grad_order2 0 --layers ${layers} --c_frames_min ${c_frames_min} --c_frames ${c_frames} --c_eps1 ${c_eps1} --c_eps2 ${c_eps2} --c_eps3 ${c_eps3}
cp -r ${outpath}/${expname} ${outpath}/test/${expname}
python ../vprocessor.py --run ${videopath}${videoname} --out ${outpath}/test/${expname} --resume 2 --port 0 --gray 1 --save_scores_only 0 --res 240x110 --day_only 0 --check_params 0 --rep 1 --all_black 0 --grad 0 --gew 1.0 --m ${features} --f ${filtersize} --init_fixed 0 --init_q 1.0 --alpha ${alpha} --beta ${beta} --gamma ${gamma} --k ${k} --theta ${theta} --eta ${eta} --rho 1 --eps1 ${eps1} --eps2 ${eps2} --eps3 ${eps3} --step_size 0 --step_adapt 0 --lambdaM ${lambdaM} --lambdaE_0 ${lambdaE_0} --lambdaE_1 ${lambdaE_1} --lambdaE_2 ${lambdaE_2} --lambdaC ${lambdaC} --blur ${blur} --grad_order2 0 --layers ${layers} --c_frames_min ${c_frames_min} --c_frames ${c_frames} --c_eps1 ${c_eps1} --c_eps2 ${c_eps2} --c_eps3 ${c_eps3}

# ---------------------------------- #
lambdaM=0

expname=lambdaM${lambdaM}"_"${videoname}"_"rep${rep}"_"m${features}"_"f${filtersize}
python ../vprocessor.py --run ${videopath}${videoname} --out ${outpath}/${expname} --port 0 --gray 1 --save_scores_only 1 --res 240x110 --day_only 0 --check_params 0 --rep ${rep} --all_black 0 --grad 0 --gew 1.0 --m ${features} --f ${filtersize} --init_fixed 0 --init_q 1.0 --alpha ${alpha} --beta ${beta} --gamma ${gamma} --k ${k} --theta ${theta} --eta ${eta} --rho ${rho} --eps1 ${eps1} --eps2 ${eps2} --eps3 ${eps3} --step_size ${step} --step_adapt 0 --lambdaM ${lambdaM} --lambdaE_0 ${lambdaE_0} --lambdaE_1 ${lambdaE_1} --lambdaE_2 ${lambdaE_2} --lambdaC ${lambdaC} --blur ${blur} --grad_order2 0 --layers ${layers} --c_frames_min ${c_frames_min} --c_frames ${c_frames} --c_eps1 ${c_eps1} --c_eps2 ${c_eps2} --c_eps3 ${c_eps3}
cp -r ${outpath}/${expname} ${outpath}/test/${expname}
python ../vprocessor.py --run ${videopath}${videoname} --out ${outpath}/test/${expname} --resume 2 --port 0 --gray 1 --save_scores_only 0 --res 240x110 --day_only 0 --check_params 0 --rep 1 --all_black 0 --grad 0 --gew 1.0 --m ${features} --f ${filtersize} --init_fixed 0 --init_q 1.0 --alpha ${alpha} --beta ${beta} --gamma ${gamma} --k ${k} --theta ${theta} --eta ${eta} --rho 1 --eps1 ${eps1} --eps2 ${eps2} --eps3 ${eps3} --step_size 0 --step_adapt 0 --lambdaM ${lambdaM} --lambdaE_0 ${lambdaE_0} --lambdaE_1 ${lambdaE_1} --lambdaE_2 ${lambdaE_2} --lambdaC ${lambdaC} --blur ${blur} --grad_order2 0 --layers ${layers} --c_frames_min ${c_frames_min} --c_frames ${c_frames} --c_eps1 ${c_eps1} --c_eps2 ${c_eps2} --c_eps3 ${c_eps3}

done
